import { TextDocument } from 'vscode-languageserver-textdocument';
import type * as ts from 'typescript';
import { ApiLanguageServiceContext } from './types';
import type * as emmet from 'vscode-emmet-helper';
import * as css from 'vscode-css-languageservice';
import * as json from 'vscode-json-languageservice';
import * as ts2 from 'vscode-typescript-languageservice';
export declare type DocumentLanguageService = ReturnType<typeof getDocumentLanguageService>;
export declare type LanguageService = ReturnType<typeof createLanguageService>;
export declare type LanguageServiceHost = ts2.LanguageServiceHost & {
    getEmmetConfig?(syntax: string): Promise<emmet.VSCodeEmmetConfig> | emmet.VSCodeEmmetConfig;
    schemaRequestService?: json.SchemaRequestService;
};
export declare type Dependencies = {
    typescript: typeof import('typescript/lib/tsserverlibrary');
};
export declare function getDocumentLanguageService({ typescript: ts }: Dependencies, getPreferences: LanguageServiceHost['getPreferences'], getFormatOptions: LanguageServiceHost['getFormatOptions']): {
    doFormatting: (document: TextDocument, options: import("vscode-languageserver-protocol").FormattingOptions) => Promise<import("vscode-languageserver-protocol").TextEdit[] | undefined>;
    getFoldingRanges: (document: TextDocument) => import("vscode-languageserver-protocol").FoldingRange[];
    doTagComplete: (document: TextDocument, position: import("vscode-languageserver-protocol").Position) => string | null | undefined;
    findLinkedEditingRanges: (document: TextDocument, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").LinkedEditingRanges | null;
};
export declare function createLanguageService({ typescript: ts }: Dependencies, vueHost: LanguageServiceHost, isTsPlugin?: boolean): {
    doValidation: (uri: string, response: (result: import("vscode-languageserver-protocol").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
    findDefinition: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").Location[] | import("vscode-languageserver-protocol").LocationLink[];
    findReferences: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").Location[];
    findTypeDefinition: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").LocationLink[];
    callHierarchy: {
        doPrepare: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").CallHierarchyItem[];
        getIncomingCalls: (item: import("vscode-languageserver-protocol").CallHierarchyItem) => import("vscode-languageserver-protocol").CallHierarchyIncomingCall[];
        getOutgoingCalls: (item: import("vscode-languageserver-protocol").CallHierarchyItem) => import("vscode-languageserver-protocol").CallHierarchyOutgoingCall[];
    };
    prepareRename: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").Range | import("vscode-jsonrpc").ResponseError<void> | undefined;
    doRename: (uri: string, position: import("vscode-languageserver-protocol").Position, newName: string) => Promise<import("vscode-languageserver-protocol").WorkspaceEdit | undefined>;
    getEditsForFileRename: (oldUri: string, newUri: string) => Promise<import("vscode-languageserver-protocol").WorkspaceEdit | undefined>;
    getSemanticTokens: (uri: string, range?: import("vscode-languageserver-protocol").Range | undefined, cancle?: import("vscode-jsonrpc").CancellationToken | undefined, resultProgress?: import("vscode-languageserver").ResultProgressReporter<import("vscode-languageserver-protocol").SemanticTokensPartialResult> | undefined) => import("vscode-languageserver-protocol").SemanticTokens | undefined;
    doHover: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").Hover | undefined;
    doComplete: (uri: string, position: import("vscode-languageserver-protocol").Position, context?: import("vscode-languageserver-protocol").CompletionContext | undefined, getNameCase?: {
        tag: () => Promise<"both" | "kebabCase" | "pascalCase">;
        attr: () => Promise<"kebabCase" | "pascalCase">;
    } | undefined) => Promise<css.CompletionList | undefined>;
    getCodeActions: (uri: string, range: import("vscode-languageserver-protocol").Range, context: import("vscode-languageserver-protocol").CodeActionContext) => Promise<import("vscode-languageserver-protocol").CodeAction[]>;
    doCodeActionResolve: (codeAction: css.CodeAction) => Promise<css.CodeAction>;
    doCompletionResolve: (item: import("vscode-languageserver-protocol").CompletionItem, newOffset?: number | undefined) => Promise<import("vscode-languageserver-protocol").CompletionItem>;
    doCodeLensResolve: (codeLens: import("vscode-languageserver-protocol").CodeLens) => import("vscode-languageserver-protocol").CodeLens;
    getSignatureHelp: (uri: string, position: import("vscode-languageserver-protocol").Position, context?: import("vscode-languageserver-protocol").SignatureHelpContext | undefined) => import("vscode-languageserver-protocol").SignatureHelp | undefined;
    getSelectionRanges: (uri: string, positions: import("vscode-languageserver-protocol").Position[]) => import("vscode-languageserver-protocol").SelectionRange[] | undefined;
    getColorPresentations: (uri: string, color: import("vscode-languageserver-protocol").Color, range: import("vscode-languageserver-protocol").Range) => import("vscode-languageserver-protocol").ColorPresentation[] | undefined;
    getCodeLens: (uri: string) => import("vscode-languageserver-protocol").CodeLens[] | undefined;
    findDocumentHighlights: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").DocumentHighlight[] | undefined;
    findDocumentSymbols: (uri: string) => import("vscode-languageserver-protocol").SymbolInformation[] | undefined;
    findDocumentLinks: (uri: string) => Promise<import("vscode-languageserver-protocol").DocumentLink[] | undefined>;
    findDocumentColors: (uri: string) => import("vscode-languageserver-protocol").ColorInformation[] | undefined;
    dispose: () => void;
    __internal__: {
        rootPath: string;
        tsPlugin: Partial<ts.LanguageService>;
        tsProgramProxy: any;
        getTsLs: (lsType: 'template' | 'script') => {
            findDefinition: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").LocationLink[];
            findTypeDefinition: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").LocationLink[];
            findReferences: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").Location[];
            prepareRename: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").Range | import("vscode-jsonrpc").ResponseError<void> | undefined;
            doRename: (uri: string, position: import("vscode-languageserver-protocol").Position, newName: string) => Promise<import("vscode-languageserver-protocol").WorkspaceEdit | undefined>;
            getEditsForFileRename: (oldUri: string, newUri: string) => Promise<import("vscode-languageserver-protocol").WorkspaceEdit | undefined>;
            getCodeActions: (uri: string, range: import("vscode-languageserver-protocol").Range, context: import("vscode-languageserver-protocol").CodeActionContext) => Promise<import("vscode-languageserver-protocol").CodeAction[] | undefined>;
            doCodeActionResolve: (codeAction: import("vscode-languageserver-protocol").CodeAction) => Promise<import("vscode-languageserver-protocol").CodeAction>;
            findDocumentHighlights: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").DocumentHighlight[];
            findDocumentSymbols: (uri: string) => import("vscode-languageserver-protocol").DocumentSymbol[];
            findWorkspaceSymbols: (uri: string) => import("vscode-languageserver-protocol").SymbolInformation[];
            doComplete: (uri: string, position: import("vscode-languageserver-protocol").Position, options?: ts.GetCompletionsAtPositionOptions | undefined) => Promise<import("vscode-languageserver-protocol").CompletionItem[]>;
            doCompletionResolve: (item: import("vscode-languageserver-protocol").CompletionItem, newOffset?: number | undefined) => Promise<import("vscode-languageserver-protocol").CompletionItem>;
            doHover: (uri: string, position: import("vscode-languageserver-protocol").Position, documentOnly?: boolean | undefined) => import("vscode-languageserver-protocol").Hover | undefined;
            doFormatting: (uri: string, options: import("vscode-languageserver-protocol").FormattingOptions, range?: import("vscode-languageserver-protocol").Range | undefined) => Promise<import("vscode-languageserver-protocol").TextEdit[]>;
            getSignatureHelp: (uri: string, position: import("vscode-languageserver-protocol").Position, context?: import("vscode-languageserver-protocol").SignatureHelpContext | undefined) => import("vscode-languageserver-protocol").SignatureHelp | undefined;
            getSelectionRange: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").SelectionRange | undefined;
            doValidation: (uri: string, options: {
                semantic?: boolean | undefined;
                syntactic?: boolean | undefined;
                suggestion?: boolean | undefined;
                declaration?: boolean | undefined;
            }, cancellationToken?: ts.CancellationToken | undefined) => import("vscode-languageserver-protocol").Diagnostic[];
            getFoldingRanges: (uri: string) => import("vscode-languageserver-protocol").FoldingRange[];
            getDocumentSemanticTokens: (uri: string, range: import("vscode-languageserver-protocol").Range, cancle?: import("vscode-jsonrpc").CancellationToken | undefined) => [number, number, number, number, number][] | undefined;
            callHierarchy: {
                doPrepare: (uri: string, position: import("vscode-languageserver-protocol").Position) => import("vscode-languageserver-protocol").CallHierarchyItem[];
                getIncomingCalls: (item: import("vscode-languageserver-protocol").CallHierarchyItem) => import("vscode-languageserver-protocol").CallHierarchyIncomingCall[];
                getOutgoingCalls: (item: import("vscode-languageserver-protocol").CallHierarchyItem) => import("vscode-languageserver-protocol").CallHierarchyOutgoingCall[];
            };
            dispose: () => void;
            __internal__: {
                raw: import("typescript/lib/tsserverlibrary").LanguageService;
                host: {
                    getImportSuggestionsCache: () => any;
                    getCompilationSettings(): ts.CompilerOptions;
                    getNewLine?(): string;
                    getProjectVersion?(): string;
                    getScriptFileNames(): string[];
                    getScriptKind?(fileName: string): ts.ScriptKind;
                    getScriptVersion(fileName: string): string;
                    getScriptSnapshot(fileName: string): ts.IScriptSnapshot | undefined;
                    getProjectReferences?(): readonly ts.ProjectReference[] | undefined;
                    getLocalizedDiagnosticMessages?(): any;
                    getCancellationToken?(): ts.HostCancellationToken;
                    getCurrentDirectory(): string;
                    getDefaultLibFileName(options: ts.CompilerOptions): string;
                    log?(s: string): void;
                    trace?(s: string): void;
                    error?(s: string): void;
                    useCaseSensitiveFileNames?(): boolean;
                    readDirectory?(path: string, extensions?: readonly string[] | undefined, exclude?: readonly string[] | undefined, include?: readonly string[] | undefined, depth?: number | undefined): string[];
                    readFile?(path: string, encoding?: string | undefined): string | undefined;
                    realpath?(path: string): string;
                    fileExists?(path: string): boolean;
                    getTypeRootsVersion?(): number;
                    resolveModuleNames?(moduleNames: string[], containingFile: string, reusedNames: string[] | undefined, redirectedReference: ts.ResolvedProjectReference | undefined, options: ts.CompilerOptions): (ts.ResolvedModule | undefined)[];
                    getResolvedModuleWithFailedLookupLocationsFromCache?(modulename: string, containingFile: string): ts.ResolvedModuleWithFailedLookupLocations | undefined;
                    resolveTypeReferenceDirectives?(typeDirectiveNames: string[], containingFile: string, redirectedReference: ts.ResolvedProjectReference | undefined, options: ts.CompilerOptions): (ts.ResolvedTypeReferenceDirective | undefined)[];
                    getDirectories?(directoryName: string): string[];
                    getCustomTransformers?(): ts.CustomTransformers | undefined;
                    isKnownTypesPackageName?(name: string): boolean;
                    installPackage?(options: ts.InstallPackageOptions): Promise<ts.ApplyCodeActionCommandResult>;
                    writeFile?(fileName: string, content: string): void;
                    getParsedCommandLine?(fileName: string): ts.ParsedCommandLine | undefined;
                    directoryExists?(directoryName: string): boolean;
                    getFormatOptions?(document: TextDocument, options?: import("vscode-languageserver-protocol").FormattingOptions | undefined): Promise<ts.FormatCodeSettings>;
                    getPreferences?(document: TextDocument): Promise<ts.UserPreferences>;
                };
                getTextDocument: (uri: string) => TextDocument | undefined;
                getValidTextDocument: (uri: string) => TextDocument | undefined;
                doCompleteSync: (uri: string, position: import("vscode-languageserver-protocol").Position, options?: ts.GetCompletionsAtPositionOptions | undefined) => import("vscode-languageserver-protocol").CompletionItem[];
            };
        };
        onInitProgress(cb: (p: number) => void): void;
        checkProject: () => boolean;
        getGlobalDocs: () => TextDocument[];
        getContext: () => ApiLanguageServiceContext;
        getD3: (document: TextDocument) => Promise<string>;
        executeCommand: (uri: string, command: string, args: any[] | undefined, connection: import("vscode-languageserver").Connection) => Promise<void>;
        detectTagNameCase: (uri: string) => {
            tag: "both" | "kebabCase" | "pascalCase" | "unsure";
            attr: "kebabCase" | "pascalCase" | "unsure";
        };
        doRefAutoClose: (document: TextDocument, position: import("vscode-languageserver-protocol").Position) => string | null | undefined;
    };
};
