import type { Position } from 'vscode-languageserver/node';
import type { TextDocument } from 'vscode-languageserver-textdocument';
import type { SourceFile } from './sourceFile';
import type { CssSourceMap, HtmlSourceMap, TeleportSourceMap, TsSourceMap } from './utils/sourceMaps';
export declare type SourceFiles = ReturnType<typeof createSourceFiles>;
export declare function createSourceFiles(): {
    getUris: () => string[];
    getAll: () => {
        uri: string;
        getTemplateTagNames: () => Set<string> | undefined;
        getTemplateAttrNames: () => Set<string> | undefined;
        getTextDocument: () => {
            readonly uri: string;
            readonly languageId: string;
            readonly version: number;
            getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
            positionAt: (offset: number) => import("vscode-languageserver-textdocument").Position;
            offsetAt: (position: import("vscode-languageserver-textdocument").Position) => number;
            readonly lineCount: number;
        };
        update: (newVueDocument: TextDocument) => {
            scriptUpdated: boolean;
            templateScriptUpdated: boolean;
        };
        updateTemplateScript: () => boolean;
        getComponentCompletionData: () => Map<string, {
            item: import("vscode-languageserver/node").CompletionItem | undefined;
            bind: import("vscode-languageserver/node").CompletionItem[];
            on: import("vscode-languageserver/node").CompletionItem[];
            slot: import("vscode-languageserver/node").CompletionItem[];
        }>;
        getDiagnostics: (response: (diags: import("vscode-languageserver/node").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
        getScriptTsDocument: () => TextDocument;
        getTsSourceMaps: () => TsSourceMap[];
        getCssSourceMaps: () => CssSourceMap[];
        getJsonSourceMaps: () => import("./utils/sourceMaps").JsonSourceMap[];
        getHtmlSourceMaps: () => HtmlSourceMap[];
        getPugSourceMaps: () => import("./utils/sourceMaps").PugSourceMap[];
        getTemplateScriptData: () => {
            projectVersion: string | undefined;
            context: string[];
            componentItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            components: string[];
            props: string[];
            setupReturns: string[];
            htmlElementItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            htmlElements: string[];
        };
        getTeleports: () => TeleportSourceMap[];
        getDescriptor: () => {
            template: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            script: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                src?: string | undefined;
            } | null;
            scriptSetup: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            styles: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                module: boolean;
                scoped: boolean;
            }[];
            customBlocks: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                type: string;
            }[];
        };
        getVueHtmlDocument: () => import("vscode-html-languageservice").HTMLDocument;
        getTemplateLsDocs: () => Map<string, TextDocument>;
        getVirtualScript: () => {
            document: TextDocument;
            sourceMap: TsSourceMap;
        };
        getScriptSetupData: () => {
            labels: (import("./parsers/types").TextRange & {
                binarys: {
                    parent: import("./parsers/types").TextRange;
                    vars: (import("./parsers/types").TextRange & {
                        isShortand: boolean;
                        inRoot: boolean;
                    })[];
                    left: import("./parsers/types").TextRange;
                    right: (import("./parsers/types").TextRange & {
                        isComputedCall: boolean;
                        withoutAs: import("./parsers/types").TextRange;
                        as: import("./parsers/types").TextRange | undefined;
                    }) | undefined;
                }[];
                label: import("./parsers/types").TextRange;
                parent: import("./parsers/types").TextRange;
            })[];
            bindings: import("./parsers/types").TextRange[];
            dollars: number[];
            withDefaultsArg: import("./parsers/types").TextRange | undefined;
            propsRuntimeArg: import("./parsers/types").TextRange | undefined;
            propsTypeArg: import("./parsers/types").TextRange | undefined;
            emitsRuntimeArg: import("./parsers/types").TextRange | undefined;
            emitsTypeArg: import("./parsers/types").TextRange | undefined;
        } | undefined;
        docLsScripts: () => {
            documents: TextDocument[];
            sourceMaps: TsSourceMap[];
        };
        getTemplateFormattingScript: () => {
            document: TextDocument | undefined;
            sourceMap: TsSourceMap | undefined;
        };
        shouldVerifyTsScript: (tsUri: string, mode: 1 | 2 | 3 | 4) => "all" | "none" | "unused";
        refs: {
            cssSourceMaps: import("@vue/reactivity").ComputedRef<CssSourceMap[]>;
            htmlSourceMap: import("@vue/reactivity").ComputedRef<HtmlSourceMap | undefined>;
            templateTsSourceMaps: import("@vue/reactivity").ComputedRef<TsSourceMap[]>;
            templateTsDocuments: import("@vue/reactivity").ComputedRef<Map<string, TextDocument>>;
            templateMainTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            templaetTsTeleports: import("@vue/reactivity").ComputedRef<TeleportSourceMap[]>;
            scriptTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            scriptTsTeleport: import("@vue/reactivity").ComputedRef<TeleportSourceMap>;
            scriptTsSourceMap: import("@vue/reactivity").ComputedRef<TsSourceMap>;
        };
    }[];
    get: (uri: string) => {
        uri: string;
        getTemplateTagNames: () => Set<string> | undefined;
        getTemplateAttrNames: () => Set<string> | undefined;
        getTextDocument: () => {
            readonly uri: string;
            readonly languageId: string;
            readonly version: number;
            getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
            positionAt: (offset: number) => import("vscode-languageserver-textdocument").Position;
            offsetAt: (position: import("vscode-languageserver-textdocument").Position) => number;
            readonly lineCount: number;
        };
        update: (newVueDocument: TextDocument) => {
            scriptUpdated: boolean;
            templateScriptUpdated: boolean;
        };
        updateTemplateScript: () => boolean;
        getComponentCompletionData: () => Map<string, {
            item: import("vscode-languageserver/node").CompletionItem | undefined;
            bind: import("vscode-languageserver/node").CompletionItem[];
            on: import("vscode-languageserver/node").CompletionItem[];
            slot: import("vscode-languageserver/node").CompletionItem[];
        }>;
        getDiagnostics: (response: (diags: import("vscode-languageserver/node").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
        getScriptTsDocument: () => TextDocument;
        getTsSourceMaps: () => TsSourceMap[];
        getCssSourceMaps: () => CssSourceMap[];
        getJsonSourceMaps: () => import("./utils/sourceMaps").JsonSourceMap[];
        getHtmlSourceMaps: () => HtmlSourceMap[];
        getPugSourceMaps: () => import("./utils/sourceMaps").PugSourceMap[];
        getTemplateScriptData: () => {
            projectVersion: string | undefined;
            context: string[];
            componentItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            components: string[];
            props: string[];
            setupReturns: string[];
            htmlElementItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            htmlElements: string[];
        };
        getTeleports: () => TeleportSourceMap[];
        getDescriptor: () => {
            template: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            script: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                src?: string | undefined;
            } | null;
            scriptSetup: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            styles: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                module: boolean;
                scoped: boolean;
            }[];
            customBlocks: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                type: string;
            }[];
        };
        getVueHtmlDocument: () => import("vscode-html-languageservice").HTMLDocument;
        getTemplateLsDocs: () => Map<string, TextDocument>;
        getVirtualScript: () => {
            document: TextDocument;
            sourceMap: TsSourceMap;
        };
        getScriptSetupData: () => {
            labels: (import("./parsers/types").TextRange & {
                binarys: {
                    parent: import("./parsers/types").TextRange;
                    vars: (import("./parsers/types").TextRange & {
                        isShortand: boolean;
                        inRoot: boolean;
                    })[];
                    left: import("./parsers/types").TextRange;
                    right: (import("./parsers/types").TextRange & {
                        isComputedCall: boolean;
                        withoutAs: import("./parsers/types").TextRange;
                        as: import("./parsers/types").TextRange | undefined;
                    }) | undefined;
                }[];
                label: import("./parsers/types").TextRange;
                parent: import("./parsers/types").TextRange;
            })[];
            bindings: import("./parsers/types").TextRange[];
            dollars: number[];
            withDefaultsArg: import("./parsers/types").TextRange | undefined;
            propsRuntimeArg: import("./parsers/types").TextRange | undefined;
            propsTypeArg: import("./parsers/types").TextRange | undefined;
            emitsRuntimeArg: import("./parsers/types").TextRange | undefined;
            emitsTypeArg: import("./parsers/types").TextRange | undefined;
        } | undefined;
        docLsScripts: () => {
            documents: TextDocument[];
            sourceMaps: TsSourceMap[];
        };
        getTemplateFormattingScript: () => {
            document: TextDocument | undefined;
            sourceMap: TsSourceMap | undefined;
        };
        shouldVerifyTsScript: (tsUri: string, mode: 1 | 2 | 3 | 4) => "all" | "none" | "unused";
        refs: {
            cssSourceMaps: import("@vue/reactivity").ComputedRef<CssSourceMap[]>;
            htmlSourceMap: import("@vue/reactivity").ComputedRef<HtmlSourceMap | undefined>;
            templateTsSourceMaps: import("@vue/reactivity").ComputedRef<TsSourceMap[]>;
            templateTsDocuments: import("@vue/reactivity").ComputedRef<Map<string, TextDocument>>;
            templateMainTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            templaetTsTeleports: import("@vue/reactivity").ComputedRef<TeleportSourceMap[]>;
            scriptTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            scriptTsTeleport: import("@vue/reactivity").ComputedRef<TeleportSourceMap>;
            scriptTsSourceMap: import("@vue/reactivity").ComputedRef<TsSourceMap>;
        };
    };
    set: (uri: string, sourceFile: SourceFile) => {
        uri: string;
        getTemplateTagNames: () => Set<string> | undefined;
        getTemplateAttrNames: () => Set<string> | undefined;
        getTextDocument: () => {
            readonly uri: string;
            readonly languageId: string;
            readonly version: number;
            getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
            positionAt: (offset: number) => import("vscode-languageserver-textdocument").Position;
            offsetAt: (position: import("vscode-languageserver-textdocument").Position) => number;
            readonly lineCount: number;
        };
        update: (newVueDocument: TextDocument) => {
            scriptUpdated: boolean;
            templateScriptUpdated: boolean;
        };
        updateTemplateScript: () => boolean;
        getComponentCompletionData: () => Map<string, {
            item: import("vscode-languageserver/node").CompletionItem | undefined;
            bind: import("vscode-languageserver/node").CompletionItem[];
            on: import("vscode-languageserver/node").CompletionItem[];
            slot: import("vscode-languageserver/node").CompletionItem[];
        }>;
        getDiagnostics: (response: (diags: import("vscode-languageserver/node").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
        getScriptTsDocument: () => TextDocument;
        getTsSourceMaps: () => TsSourceMap[];
        getCssSourceMaps: () => CssSourceMap[];
        getJsonSourceMaps: () => import("./utils/sourceMaps").JsonSourceMap[];
        getHtmlSourceMaps: () => HtmlSourceMap[];
        getPugSourceMaps: () => import("./utils/sourceMaps").PugSourceMap[];
        getTemplateScriptData: () => {
            projectVersion: string | undefined;
            context: string[];
            componentItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            components: string[];
            props: string[];
            setupReturns: string[];
            htmlElementItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            htmlElements: string[];
        };
        getTeleports: () => TeleportSourceMap[];
        getDescriptor: () => {
            template: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            script: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                src?: string | undefined;
            } | null;
            scriptSetup: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            styles: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                module: boolean;
                scoped: boolean;
            }[];
            customBlocks: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                type: string;
            }[];
        };
        getVueHtmlDocument: () => import("vscode-html-languageservice").HTMLDocument;
        getTemplateLsDocs: () => Map<string, TextDocument>;
        getVirtualScript: () => {
            document: TextDocument;
            sourceMap: TsSourceMap;
        };
        getScriptSetupData: () => {
            labels: (import("./parsers/types").TextRange & {
                binarys: {
                    parent: import("./parsers/types").TextRange;
                    vars: (import("./parsers/types").TextRange & {
                        isShortand: boolean;
                        inRoot: boolean;
                    })[];
                    left: import("./parsers/types").TextRange;
                    right: (import("./parsers/types").TextRange & {
                        isComputedCall: boolean;
                        withoutAs: import("./parsers/types").TextRange;
                        as: import("./parsers/types").TextRange | undefined;
                    }) | undefined;
                }[];
                label: import("./parsers/types").TextRange;
                parent: import("./parsers/types").TextRange;
            })[];
            bindings: import("./parsers/types").TextRange[];
            dollars: number[];
            withDefaultsArg: import("./parsers/types").TextRange | undefined;
            propsRuntimeArg: import("./parsers/types").TextRange | undefined;
            propsTypeArg: import("./parsers/types").TextRange | undefined;
            emitsRuntimeArg: import("./parsers/types").TextRange | undefined;
            emitsTypeArg: import("./parsers/types").TextRange | undefined;
        } | undefined;
        docLsScripts: () => {
            documents: TextDocument[];
            sourceMaps: TsSourceMap[];
        };
        getTemplateFormattingScript: () => {
            document: TextDocument | undefined;
            sourceMap: TsSourceMap | undefined;
        };
        shouldVerifyTsScript: (tsUri: string, mode: 1 | 2 | 3 | 4) => "all" | "none" | "unused";
        refs: {
            cssSourceMaps: import("@vue/reactivity").ComputedRef<CssSourceMap[]>;
            htmlSourceMap: import("@vue/reactivity").ComputedRef<HtmlSourceMap | undefined>;
            templateTsSourceMaps: import("@vue/reactivity").ComputedRef<TsSourceMap[]>;
            templateTsDocuments: import("@vue/reactivity").ComputedRef<Map<string, TextDocument>>;
            templateMainTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            templaetTsTeleports: import("@vue/reactivity").ComputedRef<TeleportSourceMap[]>;
            scriptTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            scriptTsTeleport: import("@vue/reactivity").ComputedRef<TeleportSourceMap>;
            scriptTsSourceMap: import("@vue/reactivity").ComputedRef<TsSourceMap>;
        };
    };
    delete: (uri: string) => boolean;
    getTsTeleports: (lsType: 'script' | 'template') => Map<string, TeleportSourceMap>;
    getTsDocuments: (lsType: 'script' | 'template') => Map<string, TextDocument>;
    getTsSourceMaps: (lsType: 'script' | 'template') => Map<string, TsSourceMap>;
    getSourceFileByTsUri: (lsType: 'script' | 'template', uri: string) => {
        uri: string;
        getTemplateTagNames: () => Set<string> | undefined;
        getTemplateAttrNames: () => Set<string> | undefined;
        getTextDocument: () => {
            readonly uri: string;
            readonly languageId: string;
            readonly version: number;
            getText: (range?: import("vscode-languageserver-textdocument").Range | undefined) => string;
            positionAt: (offset: number) => import("vscode-languageserver-textdocument").Position;
            offsetAt: (position: import("vscode-languageserver-textdocument").Position) => number;
            readonly lineCount: number;
        };
        update: (newVueDocument: TextDocument) => {
            scriptUpdated: boolean;
            templateScriptUpdated: boolean;
        };
        updateTemplateScript: () => boolean;
        getComponentCompletionData: () => Map<string, {
            item: import("vscode-languageserver/node").CompletionItem | undefined;
            bind: import("vscode-languageserver/node").CompletionItem[];
            on: import("vscode-languageserver/node").CompletionItem[];
            slot: import("vscode-languageserver/node").CompletionItem[];
        }>;
        getDiagnostics: (response: (diags: import("vscode-languageserver/node").Diagnostic[]) => void, isCancel?: (() => Promise<boolean>) | undefined) => Promise<void>;
        getScriptTsDocument: () => TextDocument;
        getTsSourceMaps: () => TsSourceMap[];
        getCssSourceMaps: () => CssSourceMap[];
        getJsonSourceMaps: () => import("./utils/sourceMaps").JsonSourceMap[];
        getHtmlSourceMaps: () => HtmlSourceMap[];
        getPugSourceMaps: () => import("./utils/sourceMaps").PugSourceMap[];
        getTemplateScriptData: () => {
            projectVersion: string | undefined;
            context: string[];
            componentItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            components: string[];
            props: string[];
            setupReturns: string[];
            htmlElementItems: {
                label: string;
                labelDetails?: {
                    parameters?: string | undefined;
                    qualifier?: string | undefined;
                    type?: string | undefined;
                } | undefined;
                kind?: import("vscode-languageserver/node").CompletionItemKind | undefined;
                tags?: 1[] | undefined;
                detail?: string | undefined;
                documentation?: string | {
                    kind: import("vscode-languageserver/node").MarkupKind;
                    value: string;
                } | undefined;
                deprecated?: boolean | undefined;
                preselect?: boolean | undefined;
                sortText?: string | undefined;
                filterText?: string | undefined;
                insertText?: string | undefined;
                insertTextFormat?: import("vscode-languageserver/node").InsertTextFormat | undefined;
                insertTextMode?: import("vscode-languageserver/node").InsertTextMode | undefined;
                textEdit?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                } | {
                    newText: string;
                    insert: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    replace: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                } | undefined;
                additionalTextEdits?: {
                    range: {
                        start: {
                            line: number;
                            character: number;
                        };
                        end: {
                            line: number;
                            character: number;
                        };
                    };
                    newText: string;
                }[] | undefined;
                commitCharacters?: string[] | undefined;
                command?: {
                    title: string;
                    command: string;
                    arguments?: any[] | undefined;
                } | undefined;
                data?: any;
            }[];
            htmlElements: string[];
        };
        getTeleports: () => TeleportSourceMap[];
        getDescriptor: () => {
            template: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            script: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                src?: string | undefined;
            } | null;
            scriptSetup: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
            } | null;
            styles: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                module: boolean;
                scoped: boolean;
            }[];
            customBlocks: {
                lang: string;
                content: string;
                loc: {
                    start: number;
                    end: number;
                };
                type: string;
            }[];
        };
        getVueHtmlDocument: () => import("vscode-html-languageservice").HTMLDocument;
        getTemplateLsDocs: () => Map<string, TextDocument>;
        getVirtualScript: () => {
            document: TextDocument;
            sourceMap: TsSourceMap;
        };
        getScriptSetupData: () => {
            labels: (import("./parsers/types").TextRange & {
                binarys: {
                    parent: import("./parsers/types").TextRange;
                    vars: (import("./parsers/types").TextRange & {
                        isShortand: boolean;
                        inRoot: boolean;
                    })[];
                    left: import("./parsers/types").TextRange;
                    right: (import("./parsers/types").TextRange & {
                        isComputedCall: boolean;
                        withoutAs: import("./parsers/types").TextRange;
                        as: import("./parsers/types").TextRange | undefined;
                    }) | undefined;
                }[];
                label: import("./parsers/types").TextRange;
                parent: import("./parsers/types").TextRange;
            })[];
            bindings: import("./parsers/types").TextRange[];
            dollars: number[];
            withDefaultsArg: import("./parsers/types").TextRange | undefined;
            propsRuntimeArg: import("./parsers/types").TextRange | undefined;
            propsTypeArg: import("./parsers/types").TextRange | undefined;
            emitsRuntimeArg: import("./parsers/types").TextRange | undefined;
            emitsTypeArg: import("./parsers/types").TextRange | undefined;
        } | undefined;
        docLsScripts: () => {
            documents: TextDocument[];
            sourceMaps: TsSourceMap[];
        };
        getTemplateFormattingScript: () => {
            document: TextDocument | undefined;
            sourceMap: TsSourceMap | undefined;
        };
        shouldVerifyTsScript: (tsUri: string, mode: 1 | 2 | 3 | 4) => "all" | "none" | "unused";
        refs: {
            cssSourceMaps: import("@vue/reactivity").ComputedRef<CssSourceMap[]>;
            htmlSourceMap: import("@vue/reactivity").ComputedRef<HtmlSourceMap | undefined>;
            templateTsSourceMaps: import("@vue/reactivity").ComputedRef<TsSourceMap[]>;
            templateTsDocuments: import("@vue/reactivity").ComputedRef<Map<string, TextDocument>>;
            templateMainTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            templaetTsTeleports: import("@vue/reactivity").ComputedRef<TeleportSourceMap[]>;
            scriptTsDocument: import("@vue/reactivity").ComputedRef<TextDocument>;
            scriptTsTeleport: import("@vue/reactivity").ComputedRef<TeleportSourceMap>;
            scriptTsSourceMap: import("@vue/reactivity").ComputedRef<TsSourceMap>;
        };
    } | undefined;
    getCssSourceMaps: () => Map<string, CssSourceMap>;
    getHtmlSourceMaps: () => Map<string, HtmlSourceMap>;
    toTsLocations: (uri: string, start: Position, end?: Position | undefined) => Generator<{
        lsType: "template" | "script";
        type: "embedded-ts";
        sourceMap: TsSourceMap;
        uri: string;
        range: {
            data: import("./utils/sourceMaps").TsMappingData;
            start: Position;
            end: Position;
        };
    } | {
        lsType: "template" | "script";
        type: "source-ts";
        uri: string;
        range: {
            start: Position;
            end: Position;
        };
        sourceMap?: undefined;
    }, void, unknown>;
    fromTsLocation: (lsType: 'script' | 'template', uri: string, start: Position, end?: Position | undefined) => Generator<{
        type: "embedded-ts";
        sourceMap: TsSourceMap;
        uri: string;
        range: {
            data: import("./utils/sourceMaps").TsMappingData;
            start: Position;
            end: Position;
        };
    } | {
        type: "source-ts";
        uri: string;
        range: {
            start: Position;
            end: Position;
        };
        sourceMap?: undefined;
    }, void, unknown>;
    fromTsLocation2: (lsType: 'script' | 'template', uri: string, start: number, end?: number | undefined) => Generator<{
        lsType: "template" | "script";
        type: "embedded-ts";
        sourceMap: TsSourceMap;
        uri: string;
        range: {
            data: import("./utils/sourceMaps").TsMappingData;
            start: number;
            end: number;
        };
    } | {
        lsType: "template" | "script";
        type: "source-ts";
        uri: string;
        range: {
            start: number;
            end: number;
        };
        sourceMap?: undefined;
    }, void, unknown>;
};
