"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const shared_1 = require("@volar/shared");
const lsTypes = ['script', 'template'];
function register({ sourceFiles, ts, getTsLs, templateTsLs, scriptTsLs, vueHost }) {
    return {
        getRootFileNames,
        emit,
        getSyntacticDiagnostics,
        getSemanticDiagnostics,
        getGlobalDiagnostics,
    };
    function getRootFileNames() {
        const set = new Set([
            ...getProgram('script').getRootFileNames().filter(fileName => { var _a, _b; return (_b = (_a = scriptTsLs.__internal__.host).fileExists) === null || _b === void 0 ? void 0 : _b.call(_a, fileName); }),
            ...getProgram('template').getRootFileNames().filter(fileName => { var _a, _b; return (_b = (_a = templateTsLs.__internal__.host).fileExists) === null || _b === void 0 ? void 0 : _b.call(_a, fileName); }),
        ]);
        return [...set.values()];
    }
    function getSyntacticDiagnostics(sourceFile, cancellationToken) {
        return lsTypes.map(lsType => transformDiagnostics(lsType, getProgram(lsType).getSyntacticDiagnostics(sourceFile, cancellationToken), 2)).flat();
    }
    function getSemanticDiagnostics(sourceFile, cancellationToken) {
        return lsTypes.map(lsType => transformDiagnostics(lsType, getProgram(lsType).getSemanticDiagnostics(sourceFile, cancellationToken), 1)).flat();
    }
    function getGlobalDiagnostics(cancellationToken) {
        return lsTypes.map(lsType => transformDiagnostics(lsType, getProgram(lsType).getGlobalDiagnostics(cancellationToken))).flat();
    }
    function emit(targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
        const scriptResult = getProgram('script').emit(targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers);
        const templateResult = getProgram('template').emit(targetSourceFile, writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers);
        return {
            emitSkipped: scriptResult.emitSkipped,
            emittedFiles: scriptResult.emittedFiles,
            diagnostics: [
                ...transformDiagnostics('script', scriptResult.diagnostics),
                ...transformDiagnostics('template', templateResult.diagnostics),
            ],
        };
    }
    function getProgram(lsType) {
        const program = (lsType === 'script' ? scriptTsLs : templateTsLs).__internal__.raw.getProgram();
        if (!program)
            throw '!program';
        return program;
    }
    // transform
    function transformDiagnostics(lsType, diagnostics, mode) {
        var _a, _b, _c, _d;
        const result = [];
        const tsLs = getTsLs(lsType);
        for (const diagnostic of diagnostics) {
            if (diagnostic.file !== undefined
                && diagnostic.start !== undefined
                && diagnostic.length !== undefined) {
                const fileName = shared_1.normalizeFileName((_c = (_b = (_a = tsLs.__internal__.host).realpath) === null || _b === void 0 ? void 0 : _b.call(_a, diagnostic.file.fileName)) !== null && _c !== void 0 ? _c : diagnostic.file.fileName);
                let checkMode = 'all';
                if (mode) {
                    const uri = shared_1.fsPathToUri(fileName);
                    const vueSourceFile = sourceFiles.getSourceFileByTsUri(lsType, uri);
                    if (vueSourceFile) {
                        checkMode = vueSourceFile.shouldVerifyTsScript(uri, mode);
                    }
                }
                if (checkMode === 'none')
                    continue;
                if (checkMode === 'unused' && !diagnostic.reportsUnnecessary)
                    continue;
                for (const tsOrVueLoc of sourceFiles.fromTsLocation2(lsType, shared_1.fsPathToUri(fileName), diagnostic.start, diagnostic.start + diagnostic.length)) {
                    if (!((_d = vueHost.fileExists) === null || _d === void 0 ? void 0 : _d.call(vueHost, shared_1.uriToFsPath(tsOrVueLoc.uri))))
                        continue;
                    if (tsOrVueLoc.type === 'embedded-ts' && !tsOrVueLoc.range.data.capabilities.diagnostic)
                        continue;
                    if (tsOrVueLoc.type === 'source-ts' && tsOrVueLoc.lsType === 'template')
                        continue;
                    let file = shared_1.uriToFsPath(tsOrVueLoc.uri) === fileName
                        ? diagnostic.file
                        : undefined;
                    if (!file) {
                        const doc = tsOrVueLoc.type === 'embedded-ts'
                            ? tsOrVueLoc.sourceMap.sourceDocument
                            : tsLs.__internal__.getTextDocument(tsOrVueLoc.uri);
                        if (doc) {
                            file = ts.createSourceFile(shared_1.uriToFsPath(tsOrVueLoc.uri), doc.getText(), tsOrVueLoc.uri.endsWith('.vue') ? ts.ScriptTarget.JSON : ts.ScriptTarget.Latest);
                        }
                    }
                    const newDiagnostic = {
                        ...diagnostic,
                        file,
                        start: tsOrVueLoc.range.start,
                        length: tsOrVueLoc.range.end - tsOrVueLoc.range.start,
                    };
                    const relatedInformation = diagnostic.relatedInformation;
                    if (relatedInformation) {
                        newDiagnostic.relatedInformation = transformDiagnostics(lsType, relatedInformation);
                    }
                    result.push(newDiagnostic);
                }
            }
            else if (diagnostic.file === undefined) {
                result.push(diagnostic);
            }
        }
        return result;
    }
}
exports.register = register;
//# sourceMappingURL=tsProgramApis.js.map